@@ -25,6 +25,7 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Types;
 import java.time.Instant;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -34,20 +35,20 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
-import static java.util.Objects.isNull;
 import java.util.Optional;
 import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import org.joda.time.DateTimeZone;
 import org.joda.time.Interval;
 import static org.sleuthkit.datamodel.BlackboardArtifact.ARTIFACT_TYPE.TSK_TL_EVENT;
 import static org.sleuthkit.datamodel.BlackboardAttribute.ATTRIBUTE_TYPE.TSK_TL_EVENT_TYPE;
 import static org.sleuthkit.datamodel.CollectionUtils.isNotEmpty;
+import static org.sleuthkit.datamodel.CommManagerSqlStringUtils.buildCSVString;
 import org.sleuthkit.datamodel.SleuthkitCase.CaseDbConnection;
 import static org.sleuthkit.datamodel.SleuthkitCase.escapeSingleQuotes;
-import static org.sleuthkit.datamodel.StringUtils.buildCSVString;
 
 /**
  * Provides access to the timeline data in a case database.
@@ -55,7 +56,7 @@ import static org.sleuthkit.datamodel.StringUtils.buildCSVString;
 public final class TimelineManager {
 
 	private static final Logger logger = Logger.getLogger(TimelineManager.class.getName());
-	
+
 	/**
 	 * Timeline event types added to the case database when it is created.
 	 */
@@ -78,19 +79,23 @@ public final class TimelineManager {
 	 */
 	private static final ImmutableList<TimelineEventType> PREDEFINED_EVENT_TYPES
 			= new ImmutableList.Builder<TimelineEventType>()
-					.add(TimelineEventType.CUSTOM_TYPES)
 					.addAll(TimelineEventType.WEB_ACTIVITY.getChildren())
 					.addAll(TimelineEventType.MISC_TYPES.getChildren())
-					.addAll(TimelineEventType.CUSTOM_TYPES.getChildren())
 					.build();
 
+	// all known artifact type ids (used for determining if an artifact is standard or custom event)
+	private static final Set<Integer> ARTIFACT_TYPE_IDS = Stream.of(BlackboardArtifact.ARTIFACT_TYPE.values())
+			.map(artType -> artType.getTypeID())
+			.collect(Collectors.toSet());
+
 	private final SleuthkitCase caseDB;
 
 	/**
-	 * Maximum timestamp to look to in future. Twelve (12) years from current date. 
+	 * Maximum timestamp to look to in future. Twelve (12) years from current
+	 * date.
 	 */
 	private static final Long MAX_TIMESTAMP_TO_ADD = Instant.now().getEpochSecond() + 394200000;
-	
+
 	/**
 	 * Mapping of timeline event type IDs to TimelineEventType objects.
 	 */
@@ -108,20 +113,25 @@ public final class TimelineManager {
 	TimelineManager(SleuthkitCase caseDB) throws TskCoreException {
 		this.caseDB = caseDB;
 
-		//initialize root and base event types, these are added to the DB in c++ land
-		ROOT_CATEGORY_AND_FILESYSTEM_TYPES.forEach(eventType -> eventTypeIDMap.put(eventType.getTypeID(), eventType));
+		List<TimelineEventType> fullList = new ArrayList<>();
+		fullList.addAll(ROOT_CATEGORY_AND_FILESYSTEM_TYPES);
+		fullList.addAll(PREDEFINED_EVENT_TYPES);
 
-		//initialize the other event types that aren't added in c++
 		caseDB.acquireSingleUserCaseWriteLock();
 		try (final CaseDbConnection con = caseDB.getConnection();
-				final Statement statement = con.createStatement()) {
-			for (TimelineEventType type : PREDEFINED_EVENT_TYPES) {
-				con.executeUpdate(statement,
-						insertOrIgnore(" INTO tsk_event_types(event_type_id, display_name, super_type_id) "
-								+ "VALUES( " + type.getTypeID() + ", '"
-								+ escapeSingleQuotes(type.getDisplayName()) + "',"
-								+ type.getParent().getTypeID()
-								+ ")")); //NON-NLS
+				final PreparedStatement pStatement = con.prepareStatement(
+						insertOrIgnore(" INTO tsk_event_types(event_type_id, display_name, super_type_id) VALUES (?, ?, ?)"),
+						Statement.NO_GENERATED_KEYS)) {
+			for (TimelineEventType type : fullList) {
+				pStatement.setLong(1, type.getTypeID());
+				pStatement.setString(2, escapeSingleQuotes(type.getDisplayName()));
+				if (type != type.getParent()) {
+					pStatement.setLong(3, type.getParent().getTypeID());
+				} else {
+					pStatement.setNull(3, java.sql.Types.INTEGER);
+				}
+
+				con.executeUpdate(pStatement);
 				eventTypeIDMap.put(type.getTypeID(), type);
 			}
 		} catch (SQLException ex) {
@@ -340,6 +350,12 @@ public final class TimelineManager {
 	 *         the event type is not found.
 	 */
 	public Optional<TimelineEventType> getEventType(long eventTypeID) {
+		// The parent EventType with ID 22 has been deprecated. This ID had two
+		// children which have be reassigned to MISC_TYPES.
+		if(eventTypeID == TimelineEventType.DEPRECATED_OTHER_EVENT_ID) {
+			return Optional.of(TimelineEventType.MISC_TYPES);
+		}
+
 		return Optional.ofNullable(eventTypeIDMap.get(eventTypeID));
 	}
 
@@ -452,32 +468,52 @@ public final class TimelineManager {
 	 * @return the event_decription_id of the inserted row.
 	 *
 	 * @throws TskCoreException
+	 * @throws DuplicateException
 	 */
-	private long addEventDescription(long dataSourceObjId, long fileObjId, Long artifactID,
+	private Long addEventDescription(long dataSourceObjId, long fileObjId, Long artifactID,
 			String fullDescription, String medDescription, String shortDescription,
-			boolean hasHashHits, boolean tagged, CaseDbConnection connection) throws TskCoreException {
-		String insertDescriptionSql
-				= "INSERT INTO tsk_event_descriptions ( "
+			boolean hasHashHits, boolean tagged, CaseDbConnection connection) throws TskCoreException, DuplicateException {
+		String tableValuesClause
+				= "tsk_event_descriptions ( "
 				+ "data_source_obj_id, content_obj_id, artifact_id,  "
 				+ " full_description, med_description, short_description, "
 				+ " hash_hit, tagged "
-				+ " ) VALUES ("
-				+ dataSourceObjId + ","
-				+ fileObjId + ","
-				+ Objects.toString(artifactID, "NULL") + ","
-				+ quotePreservingNull(fullDescription) + ","
-				+ quotePreservingNull(medDescription) + ","
-				+ quotePreservingNull(shortDescription) + ", "
-				+ booleanToInt(hasHashHits) + ","
-				+ booleanToInt(tagged)
-				+ " )";
+				+ " ) VALUES "
+				+ "(?, ?, ?, ?, ?, ?, ?, ?)";
+
+		String insertDescriptionSql = getSqlIgnoreConflict(tableValuesClause);
 
 		caseDB.acquireSingleUserCaseWriteLock();
-		try (Statement insertDescriptionStmt = connection.createStatement()) {
-			connection.executeUpdate(insertDescriptionStmt, insertDescriptionSql, PreparedStatement.RETURN_GENERATED_KEYS);
+		try {
+			PreparedStatement insertDescriptionStmt = connection.getPreparedStatement(insertDescriptionSql, PreparedStatement.RETURN_GENERATED_KEYS);
+			insertDescriptionStmt.clearParameters();
+			insertDescriptionStmt.setLong(1, dataSourceObjId);
+			insertDescriptionStmt.setLong(2, fileObjId);
+
+			if (artifactID == null) {
+				insertDescriptionStmt.setNull(3, Types.INTEGER);
+			} else {
+				insertDescriptionStmt.setLong(3, artifactID);
+			}
+
+			insertDescriptionStmt.setString(4, fullDescription);
+			insertDescriptionStmt.setString(5, medDescription);
+			insertDescriptionStmt.setString(6, shortDescription);
+			insertDescriptionStmt.setInt(7, booleanToInt(hasHashHits));
+			insertDescriptionStmt.setInt(8, booleanToInt(tagged));
+			int row = insertDescriptionStmt.executeUpdate();
+			// if no inserted rows, there is a conflict due to a duplicate event 
+			// description.  If that happens, return null as no id was inserted.
+			if (row < 1) {
+				return null;
+			}
+
 			try (ResultSet generatedKeys = insertDescriptionStmt.getGeneratedKeys()) {
-				generatedKeys.next();
-				return generatedKeys.getLong(1);
+				if (generatedKeys.next()) {
+					return generatedKeys.getLong(1);
+				} else {
+					return null;
+				}
 			}
 		} catch (SQLException ex) {
 			throw new TskCoreException("Failed to insert event description.", ex); // NON-NLS
@@ -485,6 +521,45 @@ public final class TimelineManager {
 			caseDB.releaseSingleUserCaseWriteLock();
 		}
 	}
+	
+	/**
+	 * Returns an event description id for an existing event.
+	 * 
+	 * @param dataSourceObjId	Existing data source object id
+	 * @param fileObjId			Existing content object id
+	 * @param artifactID		Existing artifact id
+	 * @param fullDescription	Full event description
+	 * @param connection		Database connection
+	 * 
+	 * @return The id of an existing description or null if none what found.
+	 * 
+	 * @throws TskCoreException 
+	 */
+	private Long getEventDescription(long dataSourceObjId, long fileObjId, Long artifactID,
+			String fullDescription, CaseDbConnection connection) throws TskCoreException {
+
+		String query = "SELECT event_description_id FROM tsk_event_descriptions "
+				+ "WHERE data_source_obj_id = " + dataSourceObjId
+				+ " AND content_obj_id = " + fileObjId
+				+ " AND artifact_id " + (artifactID != null ? " = " + artifactID : "IS null")
+				+ " AND full_description " + (fullDescription != null ? "= '"
+					+ SleuthkitCase.escapeSingleQuotes(fullDescription) + "'" : "IS null");
+
+		caseDB.acquireSingleUserCaseReadLock();
+		try (ResultSet resultSet = connection.createStatement().executeQuery(query)) {
+
+			if (resultSet.next()) {
+				long id = resultSet.getLong(1);
+				return id;
+			}
+		} catch (SQLException ex) {
+			throw new TskCoreException(String.format("Failed to get description, dataSource=%d, fileObjId=%d, artifactId=%d", dataSourceObjId, fileObjId, artifactID), ex);
+		} finally {
+			caseDB.releaseSingleUserCaseReadLock();
+		}
+
+		return null;
+	}
 
 	Collection<TimelineEvent> addEventsForNewFile(AbstractFile file, CaseDbConnection connection) throws TskCoreException {
 		Set<TimelineEvent> events = addEventsForNewFileQuiet(file, connection);
@@ -494,23 +569,24 @@ public final class TimelineManager {
 
 		return events;
 	}
-	
+
 	/**
-	 * Adds timeline events for the new file to the database.
-	 * Does not fire TSKEvents for each addition. This method should only be used if an 
-	 * update event will be sent later. For example, a data source processor may
-	 * send out a single event that a data source has been added rather than an event
+	 * Adds timeline events for the new file to the database. Does not fire
+	 * TSKEvents for each addition. This method should only be used if an update
+	 * event will be sent later. For example, a data source processor may send
+	 * out a single event that a data source has been added rather than an event
 	 * for each timeline event.
-	 * 
-	 * @param file        The new file
-	 * @param connection  Database connection to use
-	 * 
+	 *
+	 * @param file       The new file
+	 * @param connection Database connection to use
+	 *
 	 * @return Set of new events
-	 * 
-	 * @throws TskCoreException 
+	 *
+	 * @throws TskCoreException
 	 */
 	Set<TimelineEvent> addEventsForNewFileQuiet(AbstractFile file, CaseDbConnection connection) throws TskCoreException {
 		//gather time stamps into map
+		// if any of these events become deprecated in the future, filtering may need to occur.
 		Map<TimelineEventType, Long> timeMap = ImmutableMap.of(TimelineEventType.FILE_CREATED, file.getCrtime(),
 				TimelineEventType.FILE_ACCESSED, file.getAtime(),
 				TimelineEventType.FILE_CHANGED, file.getCtime(),
@@ -529,29 +605,37 @@ public final class TimelineManager {
 		Set<TimelineEvent> events = new HashSet<>();
 		caseDB.acquireSingleUserCaseWriteLock();
 		try {
-			long descriptionID = addEventDescription(file.getDataSourceObjectId(), fileObjId, null,
+			Long descriptionID = addEventDescription(file.getDataSourceObjectId(), fileObjId, null,
 					description, null, null, false, false, connection);
-
-			for (Map.Entry<TimelineEventType, Long> timeEntry : timeMap.entrySet()) {
-				Long time = timeEntry.getValue();
-				if (time > 0 && time < MAX_TIMESTAMP_TO_ADD) {// if the time is legitimate ( greater than zero and less then 12 years from current date) insert it
-					TimelineEventType type = timeEntry.getKey();
-					long eventID = addEventWithExistingDescription(time, type, descriptionID, connection);
-
-					/*
-					 * Last two flags indicating hasTags and hasHashHits are
-					 * both set to false with the assumption that this is not
-					 * possible for a new file. See JIRA-5407
-					 */
-					events.add(new TimelineEvent(eventID, descriptionID, fileObjId, null, time, type,
-							description, null, null, false, false));
-				} else {
-					if (time >= MAX_TIMESTAMP_TO_ADD) {
-						logger.log(Level.WARNING, String.format("Date/Time discarded from Timeline for %s for file %s with Id %d", timeEntry.getKey().getDisplayName(), file.getParentPath() + file.getName(), file.getId()));
-					}
-				}
+			
+			if(descriptionID == null) {
+				descriptionID = getEventDescription(file.getDataSourceObjectId(), fileObjId, null, description, connection);
 			}
+			if(descriptionID != null) {
+				for (Map.Entry<TimelineEventType, Long> timeEntry : timeMap.entrySet()) {
+					Long time = timeEntry.getValue();
+					if (time > 0 && time < MAX_TIMESTAMP_TO_ADD) {// if the time is legitimate ( greater than zero and less then 12 years from current date) insert it
+						TimelineEventType type = timeEntry.getKey();
+						long eventID = addEventWithExistingDescription(time, type, descriptionID, connection);
 
+						/*
+						 * Last two flags indicating hasTags and hasHashHits are
+						 * both set to false with the assumption that this is not
+						 * possible for a new file. See JIRA-5407
+						 */
+						events.add(new TimelineEvent(eventID, descriptionID, fileObjId, null, time, type,
+								description, null, null, false, false));
+					} else {
+						if (time >= MAX_TIMESTAMP_TO_ADD) {
+							logger.log(Level.WARNING, String.format("Date/Time discarded from Timeline for %s for file %s with Id %d", timeEntry.getKey().getDisplayName(), file.getParentPath() + file.getName(), file.getId()));
+						}
+					}
+				} 
+			} else {
+				throw new TskCoreException(String.format("Failed to get event description for file id = %d", fileObjId));
+			}
+		} catch (DuplicateException dupEx) {
+			logger.log(Level.SEVERE, "Attempt to make file event duplicate.", dupEx);
 		} finally {
 			caseDB.releaseSingleUserCaseWriteLock();
 		}
@@ -584,15 +668,19 @@ public final class TimelineManager {
 			TimelineEventType eventType;//the type of the event to add.
 			BlackboardAttribute attribute = artifact.getAttribute(new BlackboardAttribute.Type(TSK_TL_EVENT_TYPE));
 			if (attribute == null) {
-				eventType = TimelineEventType.OTHER;
+				eventType = TimelineEventType.STANDARD_ARTIFACT_CATCH_ALL;
 			} else {
 				long eventTypeID = attribute.getValueLong();
-				eventType = eventTypeIDMap.getOrDefault(eventTypeID, TimelineEventType.OTHER);
+				eventType = eventTypeIDMap.getOrDefault(eventTypeID, TimelineEventType.STANDARD_ARTIFACT_CATCH_ALL);
 			}
 
-			// @@@ This casting is risky if we change class hierarchy, but was expedient.  Should move parsing to another class
-			addArtifactEvent(((TimelineEventArtifactTypeImpl) TimelineEventType.OTHER)::makeEventDescription, eventType, artifact)
-					.ifPresent(newEvents::add);
+			try {
+				// @@@ This casting is risky if we change class hierarchy, but was expedient.  Should move parsing to another class
+				addArtifactEvent(((TimelineEventArtifactTypeImpl) TimelineEventType.STANDARD_ARTIFACT_CATCH_ALL).makeEventDescription(artifact), eventType, artifact)
+						.ifPresent(newEvents::add);
+			} catch (DuplicateException ex) {
+				logger.log(Level.SEVERE, getDuplicateExceptionMessage(artifact, "Attempt to make a timeline event artifact duplicate"), ex);
+			}
 		} else {
 			/*
 			 * If there are any event types configured to make descriptions
@@ -604,9 +692,24 @@ public final class TimelineManager {
 					.filter(eventType -> eventType.getArtifactTypeID() == artifact.getArtifactTypeID())
 					.collect(Collectors.toSet());
 
+			boolean duplicateExists = false;
 			for (TimelineEventArtifactTypeImpl eventType : eventTypesForArtifact) {
-				addArtifactEvent(eventType::makeEventDescription, eventType, artifact)
-						.ifPresent(newEvents::add);
+				try {
+					addArtifactEvent(eventType.makeEventDescription(artifact), eventType, artifact)
+							.ifPresent(newEvents::add);
+				} catch (DuplicateException ex) {
+					duplicateExists = true;
+					logger.log(Level.SEVERE, getDuplicateExceptionMessage(artifact, "Attempt to make artifact event duplicate"), ex);
+				}
+			}
+
+			// if no other timeline events were created directly, then create new 'other' ones.
+			if (!duplicateExists && newEvents.isEmpty()) {
+				try {
+					addOtherEventDesc(artifact).ifPresent(newEvents::add);
+				} catch (DuplicateException ex) {
+					logger.log(Level.SEVERE, getDuplicateExceptionMessage(artifact, "Attempt to make 'other' artifact event duplicate"), ex);
+				}
 			}
 		}
 		newEvents.stream()
@@ -616,26 +719,97 @@ public final class TimelineManager {
 	}
 
 	/**
-	 * Add an event of the given type from the given artifact. By passing the
-	 * payloadExtractor, thismethod allows a non standard description for the
-	 * given event type.
+	 * Formats a message to be displayed in response to a duplicate exception.
 	 *
-	 * @param payloadExtractor A Function that will create the decsription based
-	 *                         on the artifact. This allows the description to
-	 *                         be built based on an event type (usually OTHER)
-	 *                         different to the event type of the event.
-	 * @param eventType        The event type to create.
-	 * @param artifact         The artifact to create the event from.
+	 * @param artifact The artifact that caused the exception.
+	 * @param error    The error message to be displayed in the core of the
+	 *                 message.
+	 *
+	 * @return A formatted message (i.e.
+	 *         "[org.sleuthkit.datamodel.TimelineManager]: Attempt to make
+	 *         'other' artifact event duplicate (artifactID=12345, Source=Recent
+	 *         Activity).")
+	 */
+	private String getDuplicateExceptionMessage(BlackboardArtifact artifact, String error) {
+		String artifactIDStr = null;
+		String sourceStr = null;
+
+		if (artifact != null) {
+			artifactIDStr = Long.toString(artifact.getId());
+
+			try {
+				sourceStr = artifact.getAttributes().stream()
+					.filter(attr -> attr != null && attr.getSources() != null && !attr.getSources().isEmpty())
+					.map(attr -> String.join(",", attr.getSources()))
+					.findFirst()
+					.orElse(null);
+			} catch (TskCoreException ex) {
+				logger.log(Level.WARNING, String.format("Could not fetch artifacts for artifact id: %d.", artifact.getId()), ex);
+			}
+		}
+
+		artifactIDStr = (artifactIDStr == null) ? "<null>" : artifactIDStr;
+		sourceStr = (sourceStr == null) ? "<null>" : sourceStr;
+
+		return String.format("%s (artifactID=%s, Source=%s).", error, artifactIDStr, sourceStr);
+	}
+
+	/**
+	 * Adds 'other' type events for artifacts that have no corresponding
+	 * TimelineEventType.
+	 *
+	 * @param artifact The artifact for which to add a new timeline event.
+	 *
+	 * @return An optional of a new timeline event or empty if no time attribute
+	 *         can be determined or the artifact is null.
+	 *
+	 * @throws TskCoreException
+	 */
+	private Optional<TimelineEvent> addOtherEventDesc(BlackboardArtifact artifact) throws TskCoreException, DuplicateException {
+		if (artifact == null) {
+			return Optional.empty();
+		}
+
+		Long timeVal = artifact.getAttributes().stream()
+				.filter((attr) -> attr.getAttributeType().getValueType() == BlackboardAttribute.TSK_BLACKBOARD_ATTRIBUTE_VALUE_TYPE.DATETIME)
+				.map(attr -> attr.getValueLong())
+				.findFirst()
+				.orElse(null);
+
+		if (timeVal == null) {
+			return Optional.empty();
+		}
+
+		String description = String.format("%s: %d", artifact.getDisplayName(), artifact.getId());
+
+		TimelineEventDescriptionWithTime evtWDesc = new TimelineEventDescriptionWithTime(timeVal, description, description, description);
+
+		TimelineEventType evtType = (ARTIFACT_TYPE_IDS.contains(artifact.getArtifactTypeID()))
+				? TimelineEventType.STANDARD_ARTIFACT_CATCH_ALL
+				: TimelineEventType.CUSTOM_ARTIFACT_CATCH_ALL;
+
+		return addArtifactEvent(evtWDesc, evtType, artifact);
+	}
+
+	/**
+	 * Add an event of the given type from the given artifact to the database.
+	 *
+	 * @param eventPayload A description for this artifact including the time.
+	 * @param eventType    The event type to create.
+	 * @param artifact     The artifact to create the event from.
 	 *
 	 * @return The created event, wrapped in an Optional, or an empty Optional
 	 *         if no event was created.
 	 *
 	 * @throws TskCoreException
+	 * @throws DuplicateException
 	 */
-	private Optional<TimelineEvent> addArtifactEvent(TSKCoreCheckedFunction<BlackboardArtifact, TimelineEventDescriptionWithTime> payloadExtractor,
-			TimelineEventType eventType, BlackboardArtifact artifact) throws TskCoreException {
-		TimelineEventDescriptionWithTime eventPayload = payloadExtractor.apply(artifact);
-		if (eventPayload == null) {
+	private Optional<TimelineEvent> addArtifactEvent(TimelineEventDescriptionWithTime eventPayload,
+			TimelineEventType eventType, BlackboardArtifact artifact) throws TskCoreException, DuplicateException {
+
+		// make sure event payload is present
+		// only create event for a timeline event type if not deprecated
+		if (eventPayload == null || eventType.isDeprecated()) {
 			return Optional.empty();
 		}
 		long time = eventPayload.getTime();
@@ -651,7 +825,12 @@ public final class TimelineManager {
 		String shortDescription = eventPayload.getDescription(TimelineLevelOfDetail.LOW);
 		long artifactID = artifact.getArtifactID();
 		long fileObjId = artifact.getObjectID();
-		long dataSourceObjectID = artifact.getDataSourceObjectID();
+		Long dataSourceObjectID = artifact.getDataSourceObjectID();
+		
+		if(dataSourceObjectID == null) {
+			logger.log(Level.SEVERE, String.format("Failed to create timeline event for artifact (%d), artifact data source was null"), artifact.getId());
+			return Optional.empty();
+		}
 
 		AbstractFile file = caseDB.getAbstractFileById(fileObjId);
 		boolean hasHashHits = false;
@@ -665,15 +844,24 @@ public final class TimelineManager {
 		caseDB.acquireSingleUserCaseWriteLock();
 		try (CaseDbConnection connection = caseDB.getConnection();) {
 
-			long descriptionID = addEventDescription(dataSourceObjectID, fileObjId, artifactID,
-					fullDescription, medDescription, shortDescription,
-					hasHashHits, tagged, connection);
+			Long descriptionID = addEventDescription(dataSourceObjectID, fileObjId, artifactID,
+				fullDescription, medDescription, shortDescription,
+				hasHashHits, tagged, connection);
+			
+			if(descriptionID == null) {
+				descriptionID = getEventDescription(dataSourceObjectID, fileObjId, artifactID,
+					fullDescription, connection);
+			} 
 
-			long eventID = addEventWithExistingDescription(time, eventType, descriptionID, connection);
+			if(descriptionID != null) {
+				long eventID = addEventWithExistingDescription(time, eventType, descriptionID, connection);
 
-			event = new TimelineEvent(eventID, dataSourceObjectID, fileObjId, artifactID,
-					time, eventType, fullDescription, medDescription, shortDescription,
-					hasHashHits, tagged);
+				event = new TimelineEvent(eventID, dataSourceObjectID, fileObjId, artifactID,
+						time, eventType, fullDescription, medDescription, shortDescription,
+						hasHashHits, tagged);
+			} else {
+				throw new TskCoreException(String.format("Failed to get event description for file id = %d, artifactId %d", fileObjId, artifactID));
+			}
 
 		} finally {
 			caseDB.releaseSingleUserCaseWriteLock();
@@ -681,18 +869,37 @@ public final class TimelineManager {
 		return Optional.of(event);
 	}
 
-	private long addEventWithExistingDescription(Long time, TimelineEventType type, long descriptionID, CaseDbConnection connection) throws TskCoreException {
-		String insertEventSql
-				= "INSERT INTO tsk_events ( event_type_id, event_description_id , time) "
-				+ " VALUES (" + type.getTypeID() + ", " + descriptionID + ", " + time + ")";
+	private long addEventWithExistingDescription(Long time, TimelineEventType type, long descriptionID, CaseDbConnection connection) throws TskCoreException, DuplicateException {
+		String tableValuesClause
+				= "tsk_events ( event_type_id, event_description_id , time) VALUES (?, ?, ?)";
+
+		String insertEventSql = getSqlIgnoreConflict(tableValuesClause);
 
 		caseDB.acquireSingleUserCaseWriteLock();
-		try (Statement insertRowStmt = connection.createStatement();) {
-			connection.executeUpdate(insertRowStmt, insertEventSql, PreparedStatement.RETURN_GENERATED_KEYS);
+		try {
+			PreparedStatement insertRowStmt = connection.getPreparedStatement(insertEventSql, Statement.RETURN_GENERATED_KEYS);
+			insertRowStmt.clearParameters();
+			insertRowStmt.setLong(1, type.getTypeID());
+			insertRowStmt.setLong(2, descriptionID);
+			insertRowStmt.setLong(3, time);
+			int row = insertRowStmt.executeUpdate();
+			// if no inserted rows, return null.
+			if (row < 1) {
+				throw new DuplicateException(String.format("An event already exists in the event table for this item [time: %s, type: %s, description: %d].",
+						time == null ? "<null>" : Long.toString(time),
+						type == null ? "<null>" : type.toString(),
+						descriptionID));
+			}
 
 			try (ResultSet generatedKeys = insertRowStmt.getGeneratedKeys();) {
-				generatedKeys.next();
-				return generatedKeys.getLong(1);
+				if (generatedKeys.next()) {
+					return generatedKeys.getLong(1);
+				} else {
+					throw new DuplicateException(String.format("An event already exists in the event table for this item [time: %s, type: %s, description: %d].",
+							time == null ? "<null>" : Long.toString(time),
+							type == null ? "<null>" : type.toString(),
+							descriptionID));
+				}
 			}
 		} catch (SQLException ex) {
 			throw new TskCoreException("Failed to insert event for existing description.", ex); // NON-NLS
@@ -701,10 +908,6 @@ public final class TimelineManager {
 		}
 	}
 
-	static private String quotePreservingNull(String value) {
-		return isNull(value) ? " NULL " : "'" + escapeSingleQuotes(value) + "'";//NON-NLS
-	}
-
 	private Map<Long, Long> getEventAndDescriptionIDs(CaseDbConnection conn, long contentObjID, boolean includeArtifacts) throws TskCoreException {
 		return getEventAndDescriptionIDsHelper(conn, contentObjID, (includeArtifacts ? "" : " AND artifact_id IS NULL"));
 	}
@@ -845,7 +1048,7 @@ public final class TimelineManager {
 		if (eventDescriptionIDs.isEmpty()) {
 			return;
 		}
-		
+
 		String sql = "UPDATE tsk_event_descriptions SET tagged = " + flagValue + " WHERE event_description_id IN (" + buildCSVString(eventDescriptionIDs) + ")"; //NON-NLS
 		try (Statement updateStatement = conn.createStatement()) {
 			updateStatement.executeUpdate(sql);
@@ -872,7 +1075,7 @@ public final class TimelineManager {
 		caseDB.acquireSingleUserCaseWriteLock();
 		try (CaseDbConnection con = caseDB.getConnection(); Statement updateStatement = con.createStatement();) {
 			Map<Long, Long> eventIDs = getEventAndDescriptionIDs(con, content.getId(), true);
-			if (! eventIDs.isEmpty()) {
+			if (!eventIDs.isEmpty()) {
 				String sql = "UPDATE tsk_event_descriptions SET hash_hit = 1" + " WHERE event_description_id IN (" + buildCSVString(eventIDs.values()) + ")"; //NON-NLS
 				try {
 					updateStatement.executeUpdate(sql); //NON-NLS
@@ -1129,6 +1332,28 @@ public final class TimelineManager {
 		return result;
 	}
 
+	/**
+	 * Creates a sql statement that will do nothing due to unique constraint.
+	 *
+	 * @param insertTableValues the table, columns, and values portion of the
+	 *                          insert statement (i.e. 'table_name(col1, col2)
+	 *                          VALUES (rowVal1, rowVal2)').
+	 *
+	 * @return The sql statement.
+	 *
+	 * @throws TskCoreException
+	 */
+	private String getSqlIgnoreConflict(String insertTableValues) throws TskCoreException {
+		switch (caseDB.getDatabaseType()) {
+			case POSTGRESQL:
+				return "INSERT INTO " + insertTableValues + " ON CONFLICT DO NOTHING";
+			case SQLITE:
+				return "INSERT OR IGNORE INTO " + insertTableValues;
+			default:
+				throw new TskCoreException("Unknown DB Type: " + caseDB.getDatabaseType().name());
+		}
+	}
+
 	private String getTrueLiteral() {
 		switch (caseDB.getDatabaseType()) {
 			case POSTGRESQL:
@@ -1159,15 +1384,19 @@ public final class TimelineManager {
 	}
 
 	/**
-	 * Functional interface for a function from I to O that throws
-	 * TskCoreException.
-	 *
-	 * @param <I> Input type.
-	 * @param <O> Output type.
+	 * Exception thrown in the event of a duplicate.
 	 */
-	@FunctionalInterface
-	private interface TSKCoreCheckedFunction<I, O> {
+	private static class DuplicateException extends Exception {
 
-		O apply(I input) throws TskCoreException;
+		private static final long serialVersionUID = 1L;
+
+		/**
+		 * Main constructor.
+		 *
+		 * @param message Message for duplicate exception.
+		 */
+		DuplicateException(String message) {
+			super(message);
+		}
 	}
 }
