@@ -18,6 +18,7 @@
  */
 package org.sleuthkit.datamodel;
 
+import com.google.common.base.Strings;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.Statement;
@@ -26,12 +27,15 @@ import java.util.List;
 import java.util.Arrays;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.Map;
 import java.util.Objects;
+import java.util.Optional;
 import java.util.Queue;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import org.sleuthkit.datamodel.OsAccountManager.NotUserSIDException;
 import org.sleuthkit.datamodel.SleuthkitCase.CaseDbTransaction;
 
 /**
@@ -50,19 +54,24 @@ class TskCaseDbBridge {
     private final SleuthkitCase caseDb;
     private CaseDbTransaction trans = null;
     private final AddDataSourceCallbacks addDataSourceCallbacks;
+	private final Host imageHost;
     
     private final Map<Long, Long> fsIdToRootDir = new HashMap<>();
     private final Map<Long, TskData.TSK_FS_TYPE_ENUM> fsIdToFsType = new HashMap<>();
     private final Map<ParentCacheKey, Long> parentDirCache = new HashMap<>();
     
+    private final Map<String, OsAccount> ownerIdToAccountMap = new HashMap<>();
+	
     private static final long BATCH_FILE_THRESHOLD = 500;
     private final Queue<FileInfo> batchedFiles = new LinkedList<>();
     private final Queue<LayoutRangeInfo> batchedLayoutRanges = new LinkedList<>();
     private final List<Long> layoutFileIds = new ArrayList<>();
+    private final Map<Long, VirtualDirectory> unallocFileDirs = new HashMap<>();
     
-    TskCaseDbBridge(SleuthkitCase caseDb, AddDataSourceCallbacks addDataSourceCallbacks) {
+    TskCaseDbBridge(SleuthkitCase caseDb, AddDataSourceCallbacks addDataSourceCallbacks, Host host) {
         this.caseDb = caseDb;
         this.addDataSourceCallbacks = addDataSourceCallbacks;
+		imageHost = host;
         trans = null;
     }
     
@@ -123,6 +132,7 @@ class TskCaseDbBridge {
      * @param sha256      SHA256 hash.
      * @param deviceId    Device ID.
      * @param collectionDetails  The collection details.
+     * @param paths       Data source path(s)
      * 
      * @return The object ID of the new image or -1 if an error occurred
      */
@@ -306,6 +316,7 @@ class TskCaseDbBridge {
      * @param seq         The sequence number from fs_file->meta->seq. 
      * @param parMetaAddr The metadata address of the parent
      * @param parSeq      The parent sequence number if NTFS, -1 otherwise.
+	 * @param ownerUid	  String uid of the file owner.  May be an empty string.
      * 
      * @return 0 if successful, -1 if not
      */
@@ -319,7 +330,7 @@ class TskCaseDbBridge {
         long crtime, long ctime, long atime, long mtime,
         int meta_mode, int gid, int uid,
         String escaped_path, String extension, 
-        long seq, long parMetaAddr, long parSeq) {
+        long seq, long parMetaAddr, long parSeq, String ownerUid) {
         
         // Add the new file to the list
         batchedFiles.add(new FileInfo(parentObjId,
@@ -332,7 +343,7 @@ class TskCaseDbBridge {
                 crtime, ctime, atime, mtime,
                 meta_mode, gid, uid,
                 escaped_path, extension,
-                seq, parMetaAddr, parSeq));
+                seq, parMetaAddr, parSeq, ownerUid));
         
         // Add the current files to the database if we've exceeded the threshold or if we
         // have the root folder.
@@ -351,6 +362,46 @@ class TskCaseDbBridge {
     private long addBatchedFilesToDb() {
         List<Long> newObjIds = new ArrayList<>();
         try {
+			
+			// loop through the batch, and make sure owner accounts exist for all the files in the batch.
+			// If not, create accounts.
+			Iterator<FileInfo> it = batchedFiles.iterator();
+
+			while (it.hasNext()) {
+				FileInfo fileInfo = it.next();
+				String ownerUid = fileInfo.ownerUid;
+				if (Strings.isNullOrEmpty(fileInfo.ownerUid) == false)  { 
+					// first check the owner id is in the map, if found, then continue
+					if (this.ownerIdToAccountMap.containsKey(ownerUid)) {
+						continue;
+					}
+
+					// query the DB to get the owner account
+					try {
+						Optional<OsAccount> ownerAccount = caseDb.getOsAccountManager().getWindowsOsAccount(ownerUid, null, null, imageHost);
+						if (ownerAccount.isPresent()) {
+							// found account - add to map 
+							ownerIdToAccountMap.put(ownerUid, ownerAccount.get());
+						} else {
+							// account not found in the database,  create the account and add to map
+							// Currently we expect only NTFS systems to provide a windows style SID as owner id.
+							OsAccountManager accountMgr = caseDb.getOsAccountManager();
+							OsAccount newAccount = accountMgr.newWindowsOsAccount(ownerUid, null, null, imageHost, OsAccountRealm.RealmScope.UNKNOWN);
+							Content ds = caseDb.getContentById(fileInfo.dataSourceObjId); // Data sources are cached so this will only access the database once
+							if (ds instanceof DataSource) {
+								accountMgr.newOsAccountInstance(newAccount, (DataSource)ds, OsAccountInstance.OsAccountInstanceType.ACCESSED);
+							}
+							ownerIdToAccountMap.put(ownerUid, newAccount);
+						}
+					} catch (NotUserSIDException ex) {
+						// if the owner SID is not a user SID, set the owner account to null
+						ownerIdToAccountMap.put(ownerUid, null);
+					}
+				}
+			}
+			
+			
+					
             beginTransaction();
             FileInfo fileInfo;
             while ((fileInfo = batchedFiles.poll()) != null) {
@@ -361,6 +412,28 @@ class TskCaseDbBridge {
                         computedParentObjId = getParentObjId(fileInfo);
                     }
 
+					Long ownerAccountObjId = OsAccount.NO_ACCOUNT;
+					if (Strings.isNullOrEmpty(fileInfo.ownerUid) == false) { 
+						if (ownerIdToAccountMap.containsKey(fileInfo.ownerUid)) {
+							// for any non user SIDs, the map will have a null for account
+							if (Objects.nonNull(ownerIdToAccountMap.get(fileInfo.ownerUid))) {
+							    ownerAccountObjId = ownerIdToAccountMap.get(fileInfo.ownerUid).getId();
+							}
+						} else {
+							// Error - the map should have an account or a null at this point for the owner SID.
+							throw new TskCoreException(String.format("Failed to add file. Owner account not found for file with parent object ID: %d, name: %s, owner id: %s", fileInfo.parentObjId, fileInfo.name, fileInfo.ownerUid));
+						}
+					}
+					
+					// We've seen a case where the root folder comes in with an undefined meta type.
+					// We've also seen a case where it comes in as a regular file. The root folder should always be
+					// a directory so it will be cached properly and will not cause errors later for
+					// being an unexpected type.
+					if ((fileInfo.parentObjId == fileInfo.fsObjId)
+							&& (fileInfo.metaType != TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_DIR.getValue())) {
+						fileInfo.metaType = TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_DIR.getValue();
+					}
+					
                     long objId = addFileToDb(computedParentObjId, 
                         fileInfo.fsObjId, fileInfo.dataSourceObjId,
                         fileInfo.fsType,
@@ -371,7 +444,7 @@ class TskCaseDbBridge {
                         fileInfo.crtime, fileInfo.ctime, fileInfo.atime, fileInfo.mtime,
                         fileInfo.meta_mode, fileInfo.gid, fileInfo.uid,
                         null, TskData.FileKnown.UNKNOWN,
-                        fileInfo.escaped_path, fileInfo.extension, 
+                        fileInfo.escaped_path, fileInfo.extension, fileInfo.ownerUid, ownerAccountObjId,
                         false, trans);
                     if (fileInfo.fsObjId != fileInfo.parentObjId) {
                         // Add new file ID to the list to send to ingest unless it is the root folder
@@ -383,7 +456,7 @@ class TskCaseDbBridge {
                         fsIdToRootDir.put(fileInfo.fsObjId, objId);
                     }
 
-                    // If the file is a directory, cache the object ID
+                    // If the file is a directory, cache the object ID.
                     if ((fileInfo.metaType == TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_DIR.getValue()
                             || (fileInfo.metaType == TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_VIRT_DIR.getValue()))
                             && (fileInfo.name != null)
@@ -394,8 +467,14 @@ class TskCaseDbBridge {
                         parentDirCache.put(key, objId);
                     }
                 } catch (TskCoreException ex) {
-                    logger.log(Level.SEVERE, "Error adding file to the database - parent object ID: " + computedParentObjId
+                    if (computedParentObjId > 0) {
+                        // Most likely a database error occurred
+                        logger.log(Level.SEVERE, "Error adding file to the database - parent object ID: " + computedParentObjId
                             + ", file system object ID: " + fileInfo.fsObjId + ", name: " + fileInfo.name, ex);
+                    } else {
+                        // The parent lookup failed
+                        logger.log(Level.SEVERE, "Error adding file to the database", ex);
+                    }
                 }
             }
             commitTransaction();
@@ -436,7 +515,7 @@ class TskCaseDbBridge {
         } else {
             // There's no reason to do a database query since every folder added is being
             // stored in the cache.
-            throw new TskCoreException("Parent not found in cache (fsObjId: " +fileInfo.fsObjId + ", parMetaAddr: " + fileInfo.parMetaAddr
+            throw new TskCoreException("Could not find parent (fsObjId: " +fileInfo.fsObjId + ", parMetaAddr: " + fileInfo.parMetaAddr
                 + ", parSeq: " + fileInfo.parSeq + ", parentPath: " + parentPath + ")");
         }
     }
@@ -465,6 +544,12 @@ class TskCaseDbBridge {
             if (fsObjId == 0) {
                 fsObjIdForDb = null;
             }
+			
+            // If the layout file is in an $Unalloc folder, add the name. Otherwise use "/".
+            String parentPath = "/";
+            if (unallocFileDirs.containsKey(parentObjId)) {
+                parentPath = "/" + unallocFileDirs.get(parentObjId).getName() + "/";
+            }
             
             beginTransaction();
             long objId = addFileToDb(parentObjId, 
@@ -480,7 +565,7 @@ class TskCaseDbBridge {
                 null, null, null, null,
                 null, null, null,
                 null, TskData.FileKnown.UNKNOWN,
-                null, null, 
+                parentPath, null, null, OsAccount.NO_ACCOUNT,
                 true, trans);
             commitTransaction();
 
@@ -570,6 +655,7 @@ class TskCaseDbBridge {
             beginTransaction();
             VirtualDirectory dir = caseDb.addVirtualDirectory(fsIdToRootDir.get(fsObjId), name, trans);
             commitTransaction();
+            unallocFileDirs.put(dir.getId(), dir);
             addDataSourceCallbacks.onFilesAdded(Arrays.asList(dir.getId()));
             return dir.getId();
         } catch (TskCoreException ex) {
@@ -600,9 +686,9 @@ class TskCaseDbBridge {
         ParentCacheKey(long fsObjId, long metaAddr, long seqNum, String path) {
             this.fsObjId = fsObjId;
             this.metaAddr = metaAddr;
-            if (fsIdToFsType.containsKey(fsObjId) 
-                    && (fsIdToFsType.get(fsObjId).equals(TskData.TSK_FS_TYPE_ENUM.TSK_FS_TYPE_NTFS)
-                        || fsIdToFsType.get(fsObjId).equals(TskData.TSK_FS_TYPE_ENUM.TSK_FS_TYPE_NTFS_DETECT))) {
+            if (ownerIdToAccountMap.containsKey(fsObjId) 
+                    && (ownerIdToAccountMap.get(fsObjId).equals(TskData.TSK_FS_TYPE_ENUM.TSK_FS_TYPE_NTFS)
+                        || ownerIdToAccountMap.get(fsObjId).equals(TskData.TSK_FS_TYPE_ENUM.TSK_FS_TYPE_NTFS_DETECT))) {
                 this.seqNum = seqNum;
             } else {
                 this.seqNum = 0;
@@ -686,6 +772,7 @@ class TskCaseDbBridge {
         long seq;
         long parMetaAddr;
         long parSeq;
+		String ownerUid;
         
         FileInfo(long parentObjId, 
             long fsObjId, long dataSourceObjId,
@@ -697,7 +784,7 @@ class TskCaseDbBridge {
             long crtime, long ctime, long atime, long mtime,
             int meta_mode, int gid, int uid,
             String escaped_path, String extension, 
-            long seq, long parMetaAddr, long parSeq) {
+            long seq, long parMetaAddr, long parSeq, String ownerUid) {
             
             this.parentObjId = parentObjId;
             this.fsObjId = fsObjId;
@@ -725,6 +812,7 @@ class TskCaseDbBridge {
             this.seq = seq;
             this.parMetaAddr = parMetaAddr;
             this.parSeq = parSeq;
+			this.ownerUid = ownerUid;
         }
     }
 	
@@ -762,6 +850,8 @@ class TskCaseDbBridge {
 	 * @param known           The file known status.
 	 * @param escaped_path    The escaped path to the file.
 	 * @param extension       The file extension.
+	 * @param ownerUid        Unique id of the file owner.
+	 * @param ownerAcctObjId  Object id of the owner account.
 	 * @param hasLayout       True if this is a layout file, false otherwise.
 	 * @param transaction     The open transaction.
 	 *
@@ -779,8 +869,8 @@ class TskCaseDbBridge {
 			Long crtime, Long ctime, Long atime, Long mtime,
 			Integer meta_mode, Integer gid, Integer uid,
 			String md5, TskData.FileKnown known,
-			String escaped_path, String extension,
-			boolean hasLayout, CaseDbTransaction transaction) throws TskCoreException {
+			String escaped_path, String extension, String ownerUid, Long ownerAcctObjId,
+			boolean hasLayout,  CaseDbTransaction transaction) throws TskCoreException {
 
 		try {
 			SleuthkitCase.CaseDbConnection connection = transaction.getConnection();
@@ -788,9 +878,9 @@ class TskCaseDbBridge {
 			// Insert a row for the local/logical file into the tsk_objects table.
 			// INSERT INTO tsk_objects (par_obj_id, type) VALUES (?, ?)
 			long objectId = caseDb.addObject(parentObjId, TskData.ObjectType.ABSTRACTFILE.getObjectType(), connection);
-			
-			String fileInsert = "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path, extension, has_layout)"
-				+ " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"; // NON-NLS
+				
+			String fileInsert = "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path, extension, has_layout, owner_uid, os_account_obj_id)"
+				+ " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"; // NON-NLS
 			PreparedStatement preparedStatement = connection.getPreparedStatement(fileInsert, Statement.NO_GENERATED_KEYS);			
 			preparedStatement.clearParameters();
 			
@@ -872,6 +962,15 @@ class TskCaseDbBridge {
 			} else {
 				preparedStatement.setNull(26, java.sql.Types.INTEGER);
 			}
+			
+			preparedStatement.setString(27, ownerUid); // ownerUid
+			
+			if (ownerAcctObjId != OsAccount.NO_ACCOUNT) {
+				preparedStatement.setLong(28, ownerAcctObjId); //
+			} else {
+				preparedStatement.setNull(28, java.sql.Types.BIGINT);
+			}
+			
 			connection.executeUpdate(preparedStatement);
 
 			// If this is not a slack file create the timeline events
@@ -879,12 +978,12 @@ class TskCaseDbBridge {
 					&& TskData.TSK_DB_FILES_TYPE_ENUM.SLACK.getFileType() != fsType
 					&& (!name.equals(".")) && (!name.equals(".."))) {
 				TimelineManager timelineManager = caseDb.getTimelineManager();
-				DerivedFile derivedFile = new DerivedFile(caseDb, objectId, dataSourceObjId, name,
+				DerivedFile derivedFile = new DerivedFile(caseDb, objectId, dataSourceObjId, fsObjId, name,
 						TskData.TSK_FS_NAME_TYPE_ENUM.valueOf((short) dirType),
 						TskData.TSK_FS_META_TYPE_ENUM.valueOf((short) metaType),
 						TskData.TSK_FS_NAME_FLAG_ENUM.valueOf(dirFlags),
 						(short) metaFlags,
-						size, ctime, crtime, atime, mtime, null, null, escaped_path, null, parentObjId, null, null, extension);
+						size, ctime, crtime, atime, mtime, null, null, null, null, escaped_path, null, parentObjId, null, null, extension, ownerUid, ownerAcctObjId);
 
 				timelineManager.addEventsForNewFileQuiet(derivedFile, connection);
 			}
@@ -907,6 +1006,7 @@ class TskCaseDbBridge {
 	 * @param sha256            SHA256 hash.
 	 * @param deviceId          Device ID.
 	 * @param collectionDetails Collection details.
+	 * @param hostId            The ID of a host already in the database.
 	 * @param transaction       Case DB transaction.
 	 *
 	 * @return The newly added Image object ID.
@@ -942,13 +1042,14 @@ class TskCaseDbBridge {
 			connection.executeUpdate(preparedStatement);
 
 			// Add a row to data_source_info
-			String dataSourceInfoSql = "INSERT INTO data_source_info (obj_id, device_id, time_zone, acquisition_details) VALUES (?, ?, ?, ?)"; // NON-NLS
+			String dataSourceInfoSql = "INSERT INTO data_source_info (obj_id, device_id, time_zone, acquisition_details, host_id) VALUES (?, ?, ?, ?, ?)"; // NON-NLS
 			preparedStatement = connection.getPreparedStatement(dataSourceInfoSql, Statement.NO_GENERATED_KEYS);
 			preparedStatement.clearParameters();
 			preparedStatement.setLong(1, newObjId);
 			preparedStatement.setString(2, deviceId);
 			preparedStatement.setString(3, timezone);
 			preparedStatement.setString(4, collectionDetails);
+			preparedStatement.setLong(5, imageHost.getHostId());
 			connection.executeUpdate(preparedStatement);
 
 			return newObjId;
